﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using System.Threading;

namespace HeuristicLab.SimulationCore.Samples {
  [Item("SimpleSimulation", "A simple simulation.")]
  [Creatable("Simulations")]
  [StorableClass]
  public sealed class SimpleSimulation: Simulation {
    public override Type ProblemType {
      get { return typeof(SimpleScenario); }
    }

    private SimpleScenario Scenario {
      get {
        return Problem as SimpleScenario;
      }
    }
    
    public ValueParameter<IntValue> IterationsParameter {
      get { return (ValueParameter<IntValue>)Parameters["Iterations"]; }
    }
    
    public SimpleSimulation() : base() {
      Parameters.Add(new ValueParameter<IntValue>("Iterations", "The number of iterations.", new IntValue(100)));
    }
    [StorableConstructor]
    private SimpleSimulation(bool deserializing) : base(deserializing) { }
    private SimpleSimulation(SimpleSimulation original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new SimpleSimulation(this, cloner);
    }

    protected override void Run(CancellationToken cancellationToken) {
      IntValue counter = new IntValue(Scenario.InitialValueParameter.Value.Value);
      Results.Add(new Result("Count", counter));

      int iterations = IterationsParameter.Value.Value;
      for (int i = 0; i < iterations; i++) {
        counter.Value++;
        System.Threading.Thread.Sleep(250);
      }
    }
  }
}
